"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateSpecifierExtensions = void 0;
var _util = require("./util.cjs");
const resolveStringLiteral = path => {
  const {
    node
  } = path;
  return node.extra?.raw ?? `"${node.value}"`;
};
const resolveTemplateLiteral = (path, source) => {
  const {
    start,
    end
  } = path.node;
  return source.slice(start, end);
};
const resolveBinaryExpression = (path, source) => {
  return resolveTemplateLiteral(path, source);
};
const resolve = (path, source) => {
  if (path.isBinaryExpression({
    operator: '+'
  })) {
    const value = resolveBinaryExpression(path, source);
    const collapsed = value.replace(/['"`+)\s]|new String\(/g, '');
    return {
      value,
      relative: (0, _util.isRelative)(collapsed),
      jsExt: (0, _util.hasJsExt)(collapsed)
    };
  }
  if (path.isTemplateLiteral()) {
    const value = resolveTemplateLiteral(path, source);
    return {
      value,
      relative: (0, _util.isRelative)(value),
      jsExt: (0, _util.hasJsExt)(value)
    };
  }
  if (path.isStringLiteral()) {
    const value = resolveStringLiteral(path);
    return {
      value,
      relative: (0, _util.isRelative)(value),
      jsExt: (0, _util.hasJsExt)(value)
    };
  }
  if (path.isNewExpression() && path.get('callee').isIdentifier({
    name: 'String'
  })) {
    const newPath = path.get('arguments.0');
    return resolve(newPath, source);
  }
  return {
    value: '',
    relative: false,
    jsExt: false
  };
};
const updateSpecifierExtensions = ({
  esm,
  cjs,
  path,
  source,
  outFileExtension,
  key = 'source'
}) => {
  const property = path.get(key);
  const {
    value,
    relative,
    jsExt
  } = resolve(property, source);
  if (relative && jsExt) {
    const {
      start,
      end
    } = property.node.loc;
    esm.update(start.index, end.index, (0, _util.replaceJsExtWithOutExt)(value, outFileExtension.esm));
    cjs.update(start.index, end.index, (0, _util.replaceJsExtWithOutExt)(value, outFileExtension.cjs));
  }
};
exports.updateSpecifierExtensions = updateSpecifierExtensions;