"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateDtsSpecifiers = exports.transform = void 0;
var _promises = require("node:fs/promises");
var _core = require("@babel/core");
var _traverse = _interopRequireDefault(require("@babel/traverse"));
var _magicString = _interopRequireDefault(require("magic-string"));
var _util = require("./util.cjs");
var _helpers = require("./helpers.cjs");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const traverse = _traverse.default.default;
const transform = async (filename, opts) => {
  const source = (await (0, _promises.readFile)(filename)).toString();
  const esm = new _magicString.default(source);
  const cjs = new _magicString.default(source);
  const {
    esmPresets,
    esmPlugins,
    cjsPresets,
    comments,
    outFileExtension,
    keepFileExtension,
    ...rest
  } = opts;
  const baseOpts = {
    ...rest,
    comments,
    configFile: false
  };
  const specifierOpts = {
    esm,
    cjs,
    outFileExtension,
    source
  };
  const ast = await (0, _core.parseAsync)(source, {
    filename,
    parserOpts: {
      sourceType: 'module',
      allowAwaitOutsideFunction: true,
      plugins: ['typescript', 'jsx']
    }
  });
  if (!keepFileExtension) {
    traverse(ast, {
      ImportDeclaration(path) {
        (0, _helpers.updateSpecifierExtensions)({
          ...specifierOpts,
          path
        });
      },
      CallExpression(path) {
        if (path.get('callee').isImport()) {
          (0, _helpers.updateSpecifierExtensions)({
            ...specifierOpts,
            path,
            key: 'arguments.0'
          });
        }
      },
      'ExportNamedDeclaration|ExportAllDeclaration'(path) {
        (0, _helpers.updateSpecifierExtensions)({
          ...specifierOpts,
          path
        });
      }
    });
  }
  const {
    code: esmCode,
    map: esmMap
  } = await (0, _core.transformAsync)(esm.toString(), {
    ...baseOpts,
    generatorOpts: {
      comments
    },
    plugins: esmPlugins,
    presets: esmPresets
  });
  const {
    code: cjsCode,
    map: cjsMap
  } = await (0, _core.transformAsync)(cjs.toString(), {
    ...baseOpts,
    generatorOpts: {
      comments
    },
    presets: (0, _util.isEsModuleFile)(filename) ? esmPresets : cjsPresets
  });
  return {
    code: {
      esm: esmCode,
      cjs: cjsCode
    },
    map: {
      esm: esmMap,
      cjs: cjsMap
    }
  };
};
exports.transform = transform;
const updateDtsSpecifiers = async (filename, outFileExtension) => {
  const source = (await (0, _promises.readFile)(filename)).toString();
  const esm = new _magicString.default(source);
  const cjs = new _magicString.default(source);
  const ast = await (0, _core.parseAsync)(source, {
    filename,
    parserOpts: {
      sourceType: 'module',
      plugins: [['typescript', {
        dts: true
      }]]
    }
  });
  const opts = {
    esm,
    cjs,
    outFileExtension
  };
  traverse(ast, {
    TSImportType(path) {
      (0, _helpers.updateSpecifierExtensions)({
        ...opts,
        path,
        key: 'argument'
      });
    },
    ImportDeclaration(path) {
      (0, _helpers.updateSpecifierExtensions)({
        ...opts,
        path
      });
    },
    'ExportNamedDeclaration|ExportAllDeclaration'(path) {
      (0, _helpers.updateSpecifierExtensions)({
        ...opts,
        path
      });
    }
  });
  return {
    esm: esm.toString(),
    cjs: cjs.toString()
  };
};
exports.updateDtsSpecifiers = updateDtsSpecifiers;