"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.replaceJsExtWithOutExt = exports.logResult = exports.logHelp = exports.logError = exports.isRelative = exports.isEsModuleFile = exports.hasJsExt = exports.getRealPathAsFileUrl = exports.getOutExt = exports.getModulePresets = exports.getFiles = exports.getEsmPlugins = exports.getBabelFileHandling = exports.addDefaultPresets = void 0;
var _nodeUrl = require("node:url");
var _nodePath = require("node:path");
var _promises = require("node:fs/promises");
var _core = require("@babel/core");
const _filename = (0, _nodeUrl.fileURLToPath)(import.meta.url);
const _dirname = (0, _nodePath.dirname)(_filename);
const red = '\x1b[31m';
const cyan = '\x1b[36m';
const black = '\x1b[30m';
const jsExtRegex = /\.(js['"`\s]*?)$/i;
const relativeSpecifierRegex = /^['"`\s]*?(?:\.|\.\.)\//i;
const log = (color = cyan, msg = '', prefix = '') => {
  console.log(`${color}%s\x1b[0m`, `${prefix}${msg}`);
};
const logError = exports.logError = log.bind(null, red);
const logNotice = log.bind(null, cyan);
const logResult = exports.logResult = log.bind(null, black);
const logHelp = msg => {
  logResult(msg, '');
};
exports.logHelp = logHelp;
const getRealPathAsFileUrl = async path => {
  const realPath = await (0, _promises.realpath)(path);
  const asFileUrl = (0, _nodeUrl.pathToFileURL)(realPath).href;
  return asFileUrl;
};
exports.getRealPathAsFileUrl = getRealPathAsFileUrl;
const getFiles = async dir => {
  const dirents = await (0, _promises.readdir)(dir, {
    withFileTypes: true
  });
  const files = await Promise.all(dirents.map(dirent => {
    const path = (0, _nodePath.resolve)(dir, dirent.name);
    return dirent.isFile() ? path : getFiles(path);
  }));
  return files.flat();
};
exports.getFiles = getFiles;
const getConfigItem = (value, type = 'preset') => {
  return (0, _core.createConfigItem)(value, {
    type,
    dirname: _dirname
  });
};
const addDefaultPresets = (presets, extensions) => {
  const shouldDisallow = /\.(?:mts|cts|tsx)/i.test(extensions.join(''));
  const isTsx = extensions.includes('.tsx');
  const allExtensions = isTsx || shouldDisallow ? true : false;
  logNotice('No presets found, using default presets: ["@babel/preset-env", "@babel/preset-typescript", "@babel/preset-react"]');
  presets.push(getConfigItem('@babel/preset-env'), getConfigItem(['@babel/preset-typescript', {
    allExtensions,
    allowDeclareFields: true,
    isTSX: isTsx,
    disallowAmbiguousJSXLike: shouldDisallow
  }]), getConfigItem(['@babel/preset-react', {
    runtime: 'automatic'
  }]));
};
exports.addDefaultPresets = addDefaultPresets;
const getModulePresets = (presets, type = 'esm') => {
  const clones = [...presets];
  const modules = type === 'esm' ? false : 'commonjs';
  const envPresetIdx = clones.findIndex(preset => /@babel\/preset-env/.test(preset.file.resolved));
  if (envPresetIdx > -1) {
    clones.splice(envPresetIdx, 1, getConfigItem(['@babel/preset-env', {
      ...clones[envPresetIdx].options,
      modules
    }]));
  } else {
    clones.unshift(getConfigItem(['@babel/preset-env', {
      modules
    }]));
  }
  return clones;
};
exports.getModulePresets = getModulePresets;
const getEsmPlugins = plugins => {
  const clones = [...plugins];
  const cjsPluginIdx = clones.findIndex(plugin => /@babel\/plugin-transform-modules-commonjs/.test(plugin.file.resolved));
  if (cjsPluginIdx > -1) {
    clones.splice(cjsPluginIdx, 1);
  }
  return clones;
};
exports.getEsmPlugins = getEsmPlugins;
const hasJsExt = str => jsExtRegex.test(str);
exports.hasJsExt = hasJsExt;
const replaceJsExtWithOutExt = (str, ext) => {
  const idx = str.lastIndexOf('.js');
  if (idx > -1) {
    const step = '.js'.length;
    return `${str.substring(0, idx)}${ext}${str.substring(idx + step, str.length)}`;
  }
};
exports.replaceJsExtWithOutExt = replaceJsExtWithOutExt;
const isRelative = str => {
  return relativeSpecifierRegex.test(str) || /^`\${/i.test(str) && hasJsExt(str);
};
exports.isRelative = isRelative;
const isEsModuleFile = filename => /\.m[jt]s$/.test(filename);
exports.isEsModuleFile = isEsModuleFile;
const isCjsFile = filename => /\.c[jt]s$/.test(filename);
const getOutExt = (filename, outFileExtension, keepFileExtension, type = 'esm') => {
  if (keepFileExtension) {
    return (0, _nodePath.extname)(filename);
  }
  if (isEsModuleFile(filename) || isCjsFile(filename)) {
    return (0, _nodePath.extname)(filename).replace(/(ts)$/, 'js');
  }
  return outFileExtension[type];
};
exports.getOutExt = getOutExt;
const getBabelFileHandling = async filename => {
  const conf = await (0, _core.loadPartialConfigAsync)({
    filename,
    showIgnoredFiles: true
  });
  return conf.fileHandling;
};
exports.getBabelFileHandling = getBabelFileHandling;